%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Main parameter control
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear;
clc;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% BASE CONSTANTS

Ts = 1e-5;                  % Sample time [s]
rho = 1.225;                % Air density [kg/m^3]
Sn = 250e3;                 % Turbine rated output [VA]
fn = 50;                    % System frequency [Hz]
V1 = 690;                   % Wind turbine output voltage [V]
V2 = 400;                   % System line-to-line voltage [V]


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% WIND TURBINE SYNCHRONOUS GENERATOR PARAMETERS

% Stator Base Values
Vbase = V1*sqrt(2)/sqrt(3); % Peak value of Vabc phase voltages [V]
Ibase = (2*Sn)/(3*Vbase);   % Peak value of Iabc line currents [A]
Zbase = Vbase/Ibase;        % Stator base impedance [Ohm]
wbase = 2*pi*fn;            % Base angular frequency [rad/s]
psibase = Vbase/wbase;      % Base flux linkage [Wb]
Lbase = Zbase/wbase;        % Base stator inductance [H]

% Electrical Input Values
Pn = Sn;                    % Generator nominal power output [W]
Vll = V1;                   % Generator line-to-line voltage [V]
f = 50;                     % Generator frequency [Hz]
we = 2*pi*f;                % Rated angular frequency [rad/s]

rs = 0.005;                 % Stator resistance [pu]
xs = 0.8;                   % Stator reactance [pu]
Rs = rs*Zbase;              % Stator resistance [Ohm]
Xs = xs*Zbase;              % Stator reactance [Ohm]
Ls = Xs/we;                 % Stator inductance [H]
ls = Ls/Lbase;              % Stator inductance [pu]
T1 = ls/(wbase*rs);

psim = 2.5;                 % Stator flux linkage [Wb] (only for PMSG)
psim_pu = psim/psibase;     % Stator flux linkage [pu] (only for PMSG)

theta = 36.87;              % Initial rotor angle [deg]

% Mechanical Input Values
poles = 230;                % Number of poles
polepairs = poles/2;        % Number of pole pairs
nm = f*120/poles;           % Nominal mechanical speed [rpm]
wm = nm*pi/30;              % Nominal mechanical speed [rad/s]
Tm = Pn/wm;                 % Nominal mechanical torque [Nm]
H = 2.0;                    % Turbine inertia [s]
J = Sn*2*H/(wm^2);          % Turbine angular momentum [kg*m^2]

% Other Turbine Input
r = 26;                     % Turbine rotor radius [m]
A = pi*r^2;                 % Max area swept by rotor [m^2]
k = 0.5*A*rho;              % Constant for power calculation
Cp_max = 0.41;              % Max power coefficient value
lambda_opt = 7.95;          % Optimal tip-speed ratio for 0 pitch angle

% Constants to Aerodynamic Block
c1 = 0.5;
c2 = 116;
c3 = 0.4;
c4 = 5;
c5 = 21;
c6 = 0;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% BASE CONVERTER VALUES

fsw = 5000;                 % Switching frequency [Hz]
Tsw = 1/fsw;                % Switching period [s]
Ta = Tsw/2;                 % PWM time constant [s]
Te = 1.5*Tsw;               % Analouge to digital time constant [s]



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% SSC CONTROL PARAMETERS

% SSC Filter
rs_filter = 0.15;                           % Filter resistance [pu]
xs_filter = 0.8;                            % Filter reactance [pu]
Rs_filter = rs_filter*Zbase;                % Filter resistance [Ohm]
Xs_filter = xs_filter*Zbase;                % Filter reactance [Ohm]
Ls_filter = Xs_filter/we;                   % Filter inductance [H]
ls_filter = Ls_filter/Lbase;                % Filter inductance [pu]

% SSC Current Controller
Ti_SSC = ls_filter/(wbase*rs_filter);       % Integral time
Kp_SSC = ls_filter/(2*wbase*Te);            % Proportional gain
Ki_SSC = Kp_SSC/Ti_SSC;                     % Integral gain

% DC-Link Variables and Parameters
Vdcbase = 3*Vbase;                         
Idcbase = ((3/2)*Vbase*Ibase)/Vdcbase;
Zdcbase = Vdcbase/Idcbase;
Cdc = 0.08;
Cdc_scale = 4.52e-3;
Cdc_pu=Cdc_scale*Zdcbase*wbase*Cdc;

% DC-voltage controller
a = 3;
Teq = 2*(Te);
Tidc = (a*a*Teq);                           % Integral time
Tc = 1/(wbase*Cdc_pu);                      % Time constant                    
Kpdc = (Vdcbase*Tc)/(V2*a*Teq);             % Proportional gain
Kidc = Kpdc/Tidc;                           % Integral gain

% Reactive power controller SSC
Tf_Q_const = ls_filter/rs_filter;           % Time constant
Tiq = a*a*Teq;                              % Integral time
Kpq = Tf_Q_const*rs_filter/(a*Teq);         % Proportional gain
Kiq= Kpq/Tiq;                               % Integral gain


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% GSC CONTROL PARAMETERS

% GSC Current Controller
Ti = xs_filter/(wbase*rs_filter);           % Integral time
Kp = xs_filter/(2*wbase*Ta);                % Proportional gain
Ki = Kp/Ti;                                 % Integral gain

% GSC Speed Controller
Tiw = 4;                                    % Integral time
Kpw = 4;                                    % Proportional gain
Kiw = Kpw/Tiw;                              % Integral gain

% Pitch Angle Controller
Tb = 0.2;                                   % Integral time
Kpb = 5;                                    % Proportional gain
Kib = Kpb/Tb;                               % Integral gain
beta_min = 0;                               % Minimum pitch angle [deg]
beta_max = 90;                              % Maximum pitch angle [deg]
beta_r = 4;                                 % Maximum pitch rate [deg/s]






%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% BATTERY CONTROL PARAMETERS

% Battery Base Values
V2base = V2*sqrt(2)/sqrt(3);    % Peak value of Vabc phase voltages [V]
I2base = (2*Sn)/(3*V2base);     % Peak value of Iabc line currents [A]
Z2base = V2base/I2base;         % Stator base impedance [Ohm]
w2base = 2*pi*fn;               % Base angular frequency [rad/s]
psibase2 = V2base/w2base;       % Base flux linkage [Wb]
L2base = Z2base/w2base;         % Base stator inductance [H]


% BC Filter (same values as for SSC filter)
rs_filterb = 0.15;                      % Filter resistance [pu]
xs_filterb = 0.8;                       % Filter reactance [pu]
Rs_filterb = rs_filterb*Z2base;         % Filter resistance [Ohm]
Xs_filterb = xs_filterb*Z2base;         % Filter reactance [Ohm]
Ls_filterb = Xs_filterb/we;             % Filter inductance [H]
ls_filterb = Ls_filterb/L2base;         % Filter inductance [pu]

% Battery and Fixed Load Values
S_fixed = 80e3;                         % Fixed load power [W]
Vb = 986;                               % Battery nominal voltage [V]
Ahb = 100;                              % Battery rated capacity [Ah]
SOC0 = 60;                              % Initial SOC [%]

% Voltage controller
Tibv = ls_filterb/(wbase*rs_filterb);
Kpbv = ls_filterb/(2*wbase*Te);
Kibv = Kpbv/Tibv;